# This file is part of Tarmac.
#
# Copyright 2011 Canonical, Ltd.
#
# Tarmac is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Tarmac is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tarmac.  If not, see <http://www.gnu.org/licenses/>.
"""Handle build-dep when multiple tarmacs are running in parallel."""


from tarmac.hooks import tarmac_hooks
from tarmac.plugins import TarmacPlugin

import os
from time import sleep

TIME_BETWEEN_TWO_CHECKS = 10


class MergeDepOnFile(TarmacPlugin):
    """Tarmac plug-in for waiting on build dependency on a file existance

    This plug-in adds a timeout and then wait on every build-deps for this
    project to finish merges
    Set mergedep_on to your configuration for the current target to wait
    for this file to disappear (typically, another tarmac pid file or something
    else). Multiples files can be separated by ;
    """
    
    def run(self, command, target):
        """Wait on the existence of other files"""
        
        try:
            files = getattr(target.config, "mergedep_on").split(";")
        except AttributeError:
            return
        
        try:
            timeout = target.config.test_timeout_run
        except AttributeError:
            timeout = -1
        
        # add a timeout to let the other tarmac process starts or finish
        should_wait = True
        while(should_wait and timeout != 0):
            should_wait = False
            timeout -= 1
            sleep(TIME_BETWEEN_TWO_CHECKS)    
            for depfile in files:
                if os.path.exists(depfile):
                    self.logger.debug(u'%s exists and we merge depend on it. Waiting.' % depfile)
                    should_wait = True
                    
    
tarmac_hooks['tarmac_pre_merge'].hook(
    MergeDepOnFile(), 'Make the merges for this project waiting on a file')
